package me.unurled.raxen.commands.admin;

import static me.unurled.raxen.components.entity.player.PlayerUtils.updateSkills;
import static me.unurled.raxen.utils.Utils.colorString;
import static me.unurled.raxen.utils.Utils.colorTextComp;
import static me.unurled.raxen.utils.Utils.debug;
import static me.unurled.raxen.utils.Utils.error;
import static me.unurled.raxen.utils.Utils.msgPlayer;

import de.tr7zw.nbtapi.NBTItem;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.manager.entity.PlayerManager;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class NbtCommand implements TabExecutor {

    private final Raxen main;

    public NbtCommand(Raxen main) {
        this.main = main;
    }

    @Override
    public boolean onCommand(
        @NotNull CommandSender sender,
        @NotNull Command cmd,
        @NotNull String msg,
        @NotNull String[] args
    ) {
        if (!(sender instanceof Player player)) {
            error(
                (Raxen) Bukkit.getPluginManager().getPlugin("Raxen"),
                colorString(
                    "<red>You can't execute this command as the Console!"
                )
            );
            return true;
        }
        if (
            player.getInventory().getItemInMainHand().getType() == Material.AIR
        ) {
            msgPlayer(
                player,
                colorTextComp("<red>You must hold an Item in hand.")
            );
            return true;
        } else {
            player.getInventory().getItemInMainHand();
        }
        switch (args.length) {
            case 0, 1 -> msgPlayer(
                player,
                colorTextComp("<red>You must specify an nbt and an int.")
            );
            case 2 -> {
                debug("'" + args[0] + "'", "'" + args[1] + "'");
                ItemStack it = player.getInventory().getItemInMainHand();
                NBTItem nbti = new NBTItem(it);
                String a = "";
                if (nbti.hasTag(args[0])) {
                    a = nbti.getString(args[0]);
                }
                List<Component> lore = new ArrayList<>();
                // ItemMeta itm = it.getItemMeta();
                // lore.add((Component) itm.lore());
                boolean yes = false;
                boolean ever = false;
                for (Component ct : lore) {
                    if (ct != null) {
                        if (ct.contains(Component.text("Attributes:"))) {
                            yes = true;
                            ever = true;
                        } else {
                            if (yes) {
                                //ct.append(Component.text(colorString(attributes(args[0]) + ": " + (Integer.parseInt(args[1]) + a)))); deprecated
                                yes = false;
                            }
                        }
                    }
                }
                if (!ever) {
                    lore.add(colorTextComp("<red>Attributes:"));
                    //lore.add(colorTextComp(attributes(args[0]) + ": " + (Integer.parseInt(args[1]) + a))); deprecated
                }
                // itm.lore(lore);
                // it.setItemMeta(itm);
                PlayerManager pm =
                    (
                        (Raxen) Objects.requireNonNull(
                            Bukkit.getPluginManager().getPlugin("Raxen")
                        )
                    ).getManager()
                        .getPlayerManager();
                try {
                    nbti.setInteger(
                        args[0],
                        Integer.parseInt(args[1]) + Integer.parseInt(a)
                    );
                } catch (NumberFormatException ignored) {
                    nbti.setString(args[0], args[1]);
                }
                if (nbti.hasTag("SPEED")) {
                    pm.getSpeed().add(player, nbti.getInteger("SPEED"));
                }
                if (nbti.hasTag("HEALTH")) {
                    pm.getHealth().add(player, nbti.getInteger("HEALTH"));
                }
                if (nbti.hasTag("DEFENSE")) {
                    pm.getDefense().add(player, nbti.getInteger("DEFENSE"));
                }
                if (nbti.hasTag("STRENGTH")) {
                    pm.getStrength().add(player, nbti.getInteger("STRENGTH"));
                }
                it = nbti.getItem();
                msgPlayer(
                    player,
                    colorTextComp(
                        Raxen.getPrefix() +
                            "<white>You successfully added the nbt " +
                            /*attributes(args[0])*/args[0] +
                            "<white>with " +
                            args[1] +
                            "<white>."
                    )
                );
                updateSkills(main, player);
                player
                    .getInventory()
                    .setItem(player.getInventory().getHeldItemSlot(), it);
            }
        }
        return true;
    }

    @Override
    public @Nullable List<String> onTabComplete(
        @NotNull CommandSender sender,
        @NotNull Command command,
        @NotNull String alias,
        @NotNull String[] args
    ) {
        return null;
    }
}
