package me.unurled.raxen.commands.admin;

import static me.unurled.raxen.utils.Items.itemFrom64;
import static me.unurled.raxen.utils.Utils.colorTextComp;
import static me.unurled.raxen.utils.Utils.error;
import static me.unurled.raxen.utils.Utils.getStringFromArg;
import static me.unurled.raxen.utils.Utils.log;
import static me.unurled.raxen.utils.Utils.noPerms;
import static me.unurled.raxen.utils.Utils.setNullInArgs;
import static me.unurled.raxen.utils.Utils.validateArgs;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.entity.other.EntityNamespacedKey;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.inventory.EntityEquipment;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SpawnEntity implements TabExecutor {

    private Raxen main;
    private EntityNamespacedKey namespacedKey;

    public SpawnEntity(Raxen main) {
        this.main = main;
        this.namespacedKey = new EntityNamespacedKey(main);
    }

    @Override
    public boolean onCommand(
        @NotNull CommandSender sender,
        @NotNull Command command,
        @NotNull String label,
        @NotNull String[] args
    ) {
        if (!(sender instanceof Player)) {
            log(colorTextComp("<red>Console can't execute this command!"));
            try { // /entityspawn ZOMBIE "&cHello folks" x:0 y:0 z:0 level health defense speed strength itemStackBinary64: helmet, chestplate, leggings, boots, main-hand, off-hand
                if (validateArgs(args, 16)) {
                    args = setNullInArgs(args);
                } else {
                    log(
                        colorTextComp(
                            "<red>Please use null if you don't want some parameters in the command.</red>"
                        )
                    );
                    log(
                        colorTextComp(
                            "<red>Follow this command: /entityspawn TYPE \"name\" x y z level health defense speed strength "
                                +
                                "helmet(itemStackBinary64) chestplate(itemStackBinary64) leggings(itemStackBinary64) boots(itemStackBinary64) main-hand(itemStackBinary64) off-hand(itemStackBinary64)</red>"
                        )
                    );
                    return false;
                }

                EntityType types = EntityType.valueOf(args[0]);
                Location loc = new Location(
                    Bukkit.getWorld("world"),
                    Double.parseDouble(args[2]),
                    Double.parseDouble(args[3]),
                    Double.parseDouble(args[4])
                );
                Entity e = loc.getWorld().spawnEntity(loc, types, false);
                /* setNameSpacedKeys(
                    e,
                    getStringFromArg(args),
                    Integer.parseInt(args[6]),
                    Integer.parseInt(args[7]),
                    0,
                    Integer.parseInt(args[8]),
                    0,
                    Integer.parseInt(args[9]),
                    0,
                    Integer.parseInt(args[10]),
                    0
                ); */
                e.customName(colorTextComp(getStringFromArg(args)));
                e.setCustomNameVisible(true);
                if (e instanceof LivingEntity livingEntity) {
                    //                    livingEntity.getEquipment().set
                    ItemStack helmet = itemFrom64(args[11]);
                    ItemStack chestplate = itemFrom64(args[12]);
                    ItemStack leggins = itemFrom64(args[13]);
                    ItemStack boots = itemFrom64(args[14]);
                    ItemStack mainhand = itemFrom64(args[15]);
                    ItemStack offhand = itemFrom64(args[14]);
                    EntityEquipment equip = livingEntity.getEquipment();
                    equip.setHelmet(helmet);
                    equip.setChestplate(chestplate);
                    equip.setLeggings(leggins);
                    equip.setBoots(boots);
                    equip.setItemInMainHand(mainhand);
                    equip.setItemInOffHand(offhand);
                }

                System.out.println(e.getName());
            } catch (IllegalArgumentException e) {
                error("Something happened near the SpawnEntity command.");
                throw new RuntimeException(e);
            } catch (IOException e) {
                error(
                    "Could not deserialize item from the command EntitySpawn (command launched by "
                        +
                        sender.getName() +
                        ")"
                );
                throw new RuntimeException(e);
            }
        }
        for (String arg : args) {
            log(arg);
        }
        Player player = (Player) sender;
        if (!player.hasPermission("raxen.entityspawn.cmd")) {
            player.sendMessage(noPerms());
            return false;
        }
        try { // /entityspawn ZOMBIE "&cHello folks" x:0 y:0 z:0 level health defense speed strength itemStackBinary64: helmet, chestplate, leggings, boots, main-hand, off-hand
            if (validateArgs(args, 16)) {
                args = setNullInArgs(args);
            } else {
                log(
                    colorTextComp(
                        "<red>Please use null if you don't want some parameters in the command.</red>"
                    )
                );
                log(
                    colorTextComp(
                        "<red>Follow this command: /entityspawn TYPE \"name\" x y z level health defense speed strength "
                            +
                            "helmet(itemStackBinary64) chestplate(itemStackBinary64) leggings(itemStackBinary64) boots(itemStackBinary64) main-hand(itemStackBinary64) off-hand(itemStackBinary64)</red>"
                    )
                );
                return false;
            }

            EntityType types = EntityType.valueOf(args[0]);
            Location loc = new Location(
                Bukkit.getWorld("world"),
                Double.parseDouble(args[2]),
                Double.parseDouble(args[3]),
                Double.parseDouble(args[4])
            );
            Entity e = loc.getWorld().spawnEntity(loc, types, false);
            /* setNameSpacedKeys(
                e,
                getStringFromArg(args),
                Integer.parseInt(args[6]),
                Integer.parseInt(args[7]),
                0,
                Integer.parseInt(args[8]),
                0,
                Integer.parseInt(args[9]),
                0,
                Integer.parseInt(args[10]),
                0
            ); */
            e.customName(colorTextComp(getStringFromArg(args)));
            e.setCustomNameVisible(true);
            if (e instanceof LivingEntity livingEntity) {
                //                    livingEntity.getEquipment().set
                ItemStack helmet = itemFrom64(args[11]);
                ItemStack chestplate = itemFrom64(args[12]);
                ItemStack leggins = itemFrom64(args[13]);
                ItemStack boots = itemFrom64(args[14]);
                ItemStack mainhand = itemFrom64(args[15]);
                ItemStack offhand = itemFrom64(args[14]);
                EntityEquipment equip = livingEntity.getEquipment();
                equip.setHelmet(helmet);
                equip.setChestplate(chestplate);
                equip.setLeggings(leggins);
                equip.setBoots(boots);
                equip.setItemInMainHand(mainhand);
                equip.setItemInOffHand(offhand);
            }

            System.out.println(e.getName());
        } catch (IllegalArgumentException e) {
            error("Something happened near the SpawnEntity command.");
            throw new RuntimeException(e);
        } catch (IOException e) {
            error(
                "Could not deserialize item from the command EntitySpawn (command launched by " +
                    sender.getName() +
                    ")"
            );
            throw new RuntimeException(e);
        }

        return false;
    }

    @Override
    public @Nullable List<String> onTabComplete(
        @NotNull CommandSender sender,
        @NotNull Command command,
        @NotNull String alias,
        @NotNull String[] args
    ) {
        List<String> list = new ArrayList<>();
        for (EntityType types : EntityType.values()) {
            list.add(types.name());
        }
        return list;
    }
}
