package me.unurled.raxen.commands.admin;

import static me.unurled.raxen.utils.Utils.colorTextComp;
import static me.unurled.raxen.utils.Utils.msgPlayer;
import static me.unurled.raxen.utils.Utils.noPerms;

import java.util.ArrayList;
import java.util.List;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.tutorial.Tutorial;
import me.unurled.raxen.manager.server.TutorialManager;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class TutorialManagerCommand implements TabExecutor {

    Raxen main;
    List<String> complete = new ArrayList<>();

    public TutorialManagerCommand(Raxen main) {
        this.main = main;
    }

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command,
        @NotNull String label, @NotNull String[] args) {
        if (!sender.hasPermission("raxen.tutorial.cmd")) {
            sender.sendMessage(noPerms());
            return true;
        }
        if (args.length == 0) {
            // help
            return true;
        } else {
            TutorialManager tutorialManager = main.getManager().getTutorialManager();
            if (args.length == 1) {
                if (args[0].equalsIgnoreCase("start")) {
                    // start tutorial for sender (if sender is a player)
                    if (sender instanceof Player player) {
                        Location loc = tutorialManager
                            .generateTutorial(player);
                        tutorialManager.getTutorial(player.getUniqueId())
                            .startTutorial(player, loc);
                        return true;
                    }
                } else if (sender instanceof Player player) {
                    if (args[0].equalsIgnoreCase("stop")) {
                        // stop tutorial for sender
                        tutorialManager.getTutorial(player.getUniqueId())
                            .stopTutorial();
                        tutorialManager.removeTutorial(player.getUniqueId());
                        return true;
                    } else if (args[0].equalsIgnoreCase("next")) {
                        // next step for sender
                        tutorialManager.getTutorial(player.getUniqueId())
                            .nextStep();
                        return true;
                    } else if (args[0].equalsIgnoreCase("prev")) {
                        // prev step for sender
                        tutorialManager.getTutorial(player.getUniqueId())
                            .previousStep();
                        return true;
                    } else if (args[0].equalsIgnoreCase("status")) {
                        // status of tutorial for sender
                        Tutorial tutorial = tutorialManager.getTutorial(player.getUniqueId());
                        msgPlayer(player, colorTextComp(
                            "<gray>Tutorial status for player " + player.getName() + ": <reset>"
                                + tutorial.getStep()));
                        return true;
                    }
                }
            } else if (args.length == 2) {
                Player player = Bukkit.getPlayer(args[1]);
                if (player != null) {
                    if (player.isOnline()) {
                        if (args[0].equalsIgnoreCase("start")) {
                            // start tutorial for player
                            Location loc = tutorialManager
                                .generateTutorial(player);
                            tutorialManager.getTutorial(player.getUniqueId())
                                .startTutorial(player, loc);
                        } else if (args[0].equalsIgnoreCase("stop")) {
                            // stop tutorial for sender
                            tutorialManager.getTutorial(player.getUniqueId())
                                .stopTutorial();
                            tutorialManager.removeTutorial(player.getUniqueId());
                            return true;
                        } else if (args[0].equalsIgnoreCase("next")) {
                            // next step for sender
                            tutorialManager.getTutorial(player.getUniqueId())
                                .nextStep();
                            return true;
                        } else if (args[0].equalsIgnoreCase("prev")) {
                            // prev step for sender
                            tutorialManager.getTutorial(player.getUniqueId())
                                .previousStep();
                            return true;
                        } else if (args[0].equalsIgnoreCase("status")) {
                            // status of tutorial for sender
                            Tutorial tutorial = tutorialManager.getTutorial(player.getUniqueId());
                            msgPlayer(player, colorTextComp(
                                "<gray>Tutorial status for player " + player.getName() + ": <reset>"
                                    + tutorial.getStep()));
                            return true;
                        }
                        return true;
                    }
                }

            }
        }
        return false;
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender,
        @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if (complete.isEmpty()) {
            complete.add("start");
            complete.add("stop");
            complete.add("next");
            complete.add("prev");
            complete.add("status");
        }
        List<String> result = new ArrayList<>();
        if (args.length == 1) {
            for (String s : complete) {
                if (s.toLowerCase().startsWith(args[0].toLowerCase())) {
                    result.add(s);
                }
            }
            return result;
        } else if (args.length == 2) {
            for (Player p : Bukkit.getOnlinePlayers()) {
                if (p.getName().toLowerCase().startsWith(args[1].toLowerCase())) {
                    result.add(p.getName());
                }
            }
            return result;
        }
        return null;
    }
}
