package me.unurled.raxen.commands.player;

import java.util.ArrayList;
import java.util.List;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.entity.player.RaxenPlayer;
import me.unurled.raxen.utils.Utils;
import org.bukkit.Bukkit;
import org.bukkit.Sound;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class PartyCommand implements TabExecutor {

    Raxen main;

    List<String> complete = new ArrayList<>();

    public PartyCommand(Raxen main) {
        this.main = main;
    }

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command,
        @NotNull String label, @NotNull String[] args) {
        if (!(sender instanceof Player)) {
            Utils.errorConsoleSender(sender);
            return true;
        }
        Player player = (Player) sender;
        if (player.getWorld().getName().equalsIgnoreCase("tutorial")) {
            player.sendMessage(Utils.colorComp(
                "<grey>> <green>Sorry, you can't use this command in the tutorial!"));
            player.sendMessage(Utils.colorComp(
                "<grey>> <green>Please finish the tutorial first. Feel free to ask for help!"));
            return true;
        }
        RaxenPlayer rx = main.getManager().getPlayerManager()
            .getRaxenPlayer(player);
        if (args.length == 0) {
            player.sendMessage(
                Utils.colorComp("<reset><yellow>/party help to view all infos on party"));
            return true;
        }
        if (args[0].equalsIgnoreCase("help")) {
            player.sendMessage(
                Utils.colorComp("<reset><yellow>/party help to view all infos on party"));
            player.sendMessage(Utils.colorComp("<reset><yellow>/party create to create a party"));
            player.sendMessage(Utils.colorComp(
                "<reset><yellow>/party invite <player> to invite a player to your party"));
            player.sendMessage(Utils.colorComp("<reset><yellow>/party leave to leave your party"));
            player.sendMessage(Utils.colorComp(
                "<reset><yellow>/party kick <player> to kick a player from your party"));
            player.sendMessage(
                Utils.colorComp("<reset><yellow>/party disband to disband your party"));
            player.sendMessage(
                Utils.colorComp("<reset><yellow>/party accept to accept a party invite"));
            player.sendMessage(
                Utils.colorComp("<reset><yellow>/party decline to deny a party invite"));
            player.sendMessage(
                Utils.colorComp("<reset><yellow>/party list to list all members of your party"));
            player.sendMessage(
                Utils.colorComp("<reset><yellow>\\ before every message to send it to your party"));
            return true;
        } else if (args[0].equalsIgnoreCase("create")) {
            main.getManager().getPartyManager().createParty(player, rx);
            return true;
        } else if (args[0].equalsIgnoreCase("invite")) {
            if (rx.getParty() != null) {
                if (args.length != 2) {
                    player.sendMessage(Utils.colorComp(
                        "<grey>> <green>Usage: /party invite <player>"));
                    return true;
                } else {
                    String name = args[1];
                    Player target = main.getServer().getPlayer(name);
                    RaxenPlayer rx2 = main.getManager().getPlayerManager()
                        .getRaxenPlayer(target);
                    if (rx2 == null || target == null || !target.isOnline()) {
                        player.sendMessage(Utils.colorComp(
                            "<grey>> <green>Player not found!"));
                        return true;
                    }
                    if (rx.getParty().isFull()) {
                        player.sendMessage(Utils.colorComp(
                            "<grey>> <green>Your party is full!"));
                        return true;
                    } else {
                        rx.getParty().sendInvite(target, rx2, player);
                    }
                }
            } else {
                player.sendMessage(Utils.colorComp(
                    "<grey>> <green>You are not in a party!"));
                return true;
            }
        } else if (args[0].equalsIgnoreCase("accept")) {
            if (rx.getInvitedParty() == null) {
                player.sendMessage(Utils.colorComp(
                    "<grey>> <green>You have no pending party invites!"));
                return true;
            } else {
                rx.getInvitedParty().addPlayer(player);
                rx.setParty(rx.getInvitedParty());
                rx.setInvitedParty(null);
                return true;
            }
        } else if (args[0].equalsIgnoreCase("decline")) {
            if (rx.getInvitedParty() != null) {
                rx.getInvitedParty().sendMessage(Utils.colorComp(
                    "<grey>> <green>" + player.getName() + " has declined the party invite!"));
            }
            rx.setInvitedParty(null);
            player.sendMessage(Utils.colorComp(
                "<grey>> <green>You have declined the party invite!"));
        } else if (args[0].equalsIgnoreCase("leave")) {
            if (rx.getParty() == null) {
                player.sendMessage(Utils.colorComp(
                    "<grey>> <green>You are not in a party!"));
                return true;
            } else {
                rx.getParty().leavePlayer(player.getUniqueId(), player.getName());
                rx.setParty(null);
                return true;
            }
        } else if (args[0].equalsIgnoreCase("kick")) {
            if (rx.getParty() == null) {
                player.sendMessage(Utils.colorComp(
                    "<grey>> <green>You are not in a party!"));
                return true;
            } else {
                if (!rx.getParty().isLeader(player)) {
                    player.sendMessage(Utils.colorComp(
                        "<grey>> <green>You are not the party leader!"));
                    return true;
                } else {
                    if (args.length != 2) {
                        player.sendMessage(Utils.colorComp(
                            "<grey>> <green>Usage: /party kick <player>"));
                        return true;
                    } else {
                        String name = args[1];
                        Player target = main.getServer().getPlayer(name);
                        RaxenPlayer rx2 = main.getManager().getPlayerManager()
                            .getRaxenPlayer(target);
                        if (rx2 == null || target == null || !target.isOnline()) {
                            player.sendMessage(Utils.colorComp(
                                "<grey>> <green>Player not found!"));
                            return true;
                        }
                        if (!rx2.getParty().equals(rx.getParty())) {
                            player.sendMessage(Utils.colorComp(
                                "<grey>> <green>This player is not in your party!"));
                            return true;
                        }
                        if (target.equals(player) || rx.equals(rx2)) {
                            player.sendMessage(Utils.colorComp(
                                "<grey>> <green>You can't kick yourself!"));
                            return true;
                        }
                        target.playSound(target.getLocation(), Sound.ENTITY_EXPERIENCE_ORB_PICKUP,
                            1, 1);
                        rx.getParty().leavePlayer(target.getUniqueId(), target.getName());
                        return true;
                    }
                }
            }
        } else {
            if (rx.getParty() != null) {
                StringBuilder sb = new StringBuilder();
                for (String s : args) {
                    sb.append(s).append(" ");
                }
                String message = sb.toString().trim();
                rx.getParty().sendMessage(player, Utils.colorComp(
                    "<reset>" + message));
            } else {
                player.sendMessage(Utils.colorComp(
                    "<grey>> <green>Not valid command!"));
                player.sendMessage(Utils.colorComp(
                    "<grey>> <green>/party help to view all infos on party"));
                return true;
            }
        }
        return true;
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender,
        @NotNull Command command, @NotNull String label, @NotNull String[] args) {

        if (complete.isEmpty()) {
            complete.add("create");
            complete.add("invite");
            complete.add("accept");
            complete.add("decline");
            complete.add("leave");
            complete.add("kick");
            complete.add("help");
        }
        List<String> result = new ArrayList<>();
        if (args.length == 1) {
            for (String s : complete) {
                if (s.toLowerCase().startsWith(args[0].toLowerCase())) {
                    result.add(s);
                }
            }
            return result;
        } else if (args.length == 2) {
            for (Player p : Bukkit.getOnlinePlayers()) {
                if (p.getName().toLowerCase().startsWith(args[1].toLowerCase())) {
                    result.add(p.getName());
                }
            }
            return result;
        }

        return null;
    }
}
