package me.unurled.raxen.components.entity.other;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.UUID;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.entity.Attributes.Attribute;
import me.unurled.raxen.components.entity.Attributes.Health;
import me.unurled.raxen.components.entity.DamageType;
import me.unurled.raxen.components.entity.player.RaxenPlayer;
import me.unurled.raxen.manager.entity.EntityManager;
import me.unurled.raxen.utils.EntityUtils;
import me.unurled.raxen.utils.Tags;
import me.unurled.raxen.utils.Utils;
import net.kyori.adventure.text.Component;
import net.minecraft.world.entity.Entity;
import org.bukkit.Bukkit;
import org.bukkit.EntityEffect;
import org.bukkit.Location;
import org.bukkit.Sound;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;

public class MobData {

    public Raxen main;
    private static int MOB_ID = 0;
    public int id = MOB_ID++;
    public boolean dead = false;
    public LivingEntity entity;
    public LivingEntity mount = null;

    String nameWithoutLevel;
    String fullName;

    public MobType mobType;

    public Attribute attributes;
    public HashMap<UUID, Long> lastAttackedByTimes;
    public HashMap<UUID, Long> damageDealt;
    public long lastDamaged = 0;
    public long lastKnockback = 0;

    public boolean despawned = false;
    public long invuln = 0; // cannot attack or damage
    private Runnable lastScheduledRevertHP = null;

    // TODO: public ArrayList<MobAttributes> attributes;

    // TODO: mob ai

    // TODO: mob spawn

    public MobData() {
        lastAttackedByTimes = new HashMap<>();
        damageDealt = new HashMap<>();
    }

    public void mount(Class<? extends Entity> mountClass) {
        if (entity == null || dead) {
            return;
        }
        mount = EntityUtils.createLivingEntity(mountClass, entity.getLocation());
        mount.addPassenger(entity);

    }

    public void die() {
        die(true);
    }

    public void die(boolean drop) {
        if (dead) {
            return;
        }
        dead = true;
        Location deathLoc = entity.getLocation();
        entity.customName(Component.text(fullName));
        Tags.makeFloatingText("<gold>+" + mobType.xp + " XP", deathLoc, 1.3, 1.0, 1.5, 1);
        if (mount != null) {
            try {
                mount.eject();
                entity.leaveVehicle();
            } catch (Exception e) {
                e.printStackTrace();
            }
            mount.damage(1000000000);
        }
        entity.damage(1000000000);
        new BukkitRunnable() {
            @Override
            public void run() {
                despawn();
            }
        }.runTaskLater(main, 20);
        int totalLevel = 0;
        int pdCount = 0;
        ArrayList<RaxenPlayer> raxenPlayers = new ArrayList<>();
        double damageSum = 0;
        RaxenPlayer rx = null;
        for (Entry<UUID, Long> entry : damageDealt.entrySet()) {
            if ((rx = main.getManager().getPlayerManager().getRaxenPlayer(entry.getKey()))
                != null) {
                raxenPlayers.add(rx);
                damageSum += entry.getValue();
                totalLevel += rx.getLevel();
                pdCount++;
            }
        }
        if (pdCount < 1) {
            totalLevel = 1;
            pdCount = 1;
        }
        double avgLv = totalLevel / (double) pdCount;
        for (RaxenPlayer tx : raxenPlayers) {
            if (damageDealt.containsKey(tx.getPlayer().getUniqueId())) {
                tx.incrementMobCounter(mobType.identifier);
                double precentage =
                    damageDealt.get(tx.getPlayer().getUniqueId()) / (double) damageSum;
                if (precentage >= 0.1) {
                    tx.addMobKills();
                }
                long amt = (long) Math.ceil(mobType.xp * precentage);
                if (!(mobType.level >= 80 && tx.getLevel() >= 80) && Math.abs(
                    tx.getLevel() - mobType.level) >= 10) {
                    double penalty = 0.7;
                    penalty -= (Math.abs(tx.getLevel() - mobType.level) - 10) * 0.06;
                    if (penalty < 0.1) {
                        penalty = 0.1;
                    }
                    tx.gainXp((long) Math.ceil(amt * penalty), true);
                } else {
                    tx.gainXp(amt);
                }
            }
        }
        long mostDealt = 0;
        UUID priority = null;
        RaxenPlayer topDamage = null;
        for (Entry<UUID, Long> entry : damageDealt.entrySet()) {
            Player p = Bukkit.getPlayer(entry.getKey());
            if (p == null) {
                continue;
            }
            if (me.unurled.raxen.utils.Math.flatDistance(p.getLocation(), deathLoc) < 50) {
                if (entry.getValue() >= mostDealt) {
                    if (entry.getValue() == mostDealt && Math.random() > 0.5) {
                        continue;
                    }
                    if (main.getManager().getPlayerManager().getRaxenPlayer(entry.getKey())
                        == null) {
                        continue;
                    }
                    mostDealt = entry.getValue();
                    priority = entry.getKey();
                    topDamage = main.getManager().getPlayerManager().getRaxenPlayer(priority);
                }
            }
        }
        EntityManager manager = main.getManager().getEntityManager();
        if (drop || manager.getHealth().get(entity) < manager.getMaxMana().get(entity)) {
            // TODO: mobdrop
        }

        // TODO: respawn
    }

    public void despawn() {
        if (despawned) {
            return;
        }
        // TODO: respawn
        despawned = true;
        dead = true;
        EntityManager manager = main.getManager().getEntityManager();
        if (entity == null) {
            manager.getSpawnedMob().remove(entity.getUniqueId());
            manager.getSpawnedMobsOnlyMain().remove(entity.getUniqueId());
            entity.remove();
        }
        entity = null;
        if (mount != null) {
            manager.getSpawnedMob().remove(mount.getUniqueId());
            manager.getSpawnedMobsOnlyMain().remove(mount.getUniqueId());
            mount.remove();
        }
        mount = null;
        // TODO: ai
        if (damageDealt != null) {
            damageDealt = null;
        }
        attributes = null;
        lastAttackedByTimes = null;
    }

    public boolean attack(RaxenPlayer rx) {
        return attack(rx, 0.5, -1, false);
    }

    public boolean attack(RaxenPlayer pd, int rpgDamage) {
        return attack(pd, 0.5, rpgDamage, false);
    }

    public boolean attack(RaxenPlayer pd, int rpgDamage, boolean projectile) {
        return attack(pd, 0.5, rpgDamage, projectile);
    }

    public int getDamage() {
        return main.getManager().getEntityManager().getStrength()
            .get(entity); // TODO: get damage properly
    }

    public boolean attack(RaxenPlayer pd, double knockback, int rpgDamage, boolean projectile) {
        if (System.currentTimeMillis() < invuln) {
            return false;
        }
        boolean success = false;
        success = pd.damage(rpgDamage > -1 ? rpgDamage : getDamage(), entity,
            DamageType.NORMAL);
        if (success && !projectile) {
            pd.knockback(entity, knockback);
        }
        return success;
    }

    public void knockback(LivingEntity attacker, double knockBackMultiplier) {
        return;
    }


    public boolean damage(int damageAmount, org.bukkit.entity.Entity damager,
        DamageType damageType) {
        return damage(damageAmount, damager, damageType, false);
    }

    public boolean damage(int damageAmount, org.bukkit.entity.Entity damager, DamageType damageType,
        boolean crit) {
        if (System.currentTimeMillis() < invuln) {
            return false;
        }
        if (dead) {
            return false;
        }
        if (damager != null && damager instanceof Player player) {
            if (!player.isOnline()
                || main.getManager().getPlayerManager().getRaxenPlayer(player) == null) {
                return false;
            }
        }
        Health health = main.getManager().getEntityManager().getHealth();
        if (damageAmount > health.get(entity)) {
            damageAmount = health.get(entity);
        }
        if (damager != null && damager instanceof Player player) {
            if (damageDealt.containsKey(damager.getUniqueId())) {
                damageDealt.put(damager.getUniqueId(),
                    damageDealt.get(damager.getUniqueId()) + damageAmount);
            } else {
                damageDealt.put(damager.getUniqueId(), (long) damageAmount);
            }
            // TODO: AI stuff
        }
        if (damager != null) {
            lastAttackedByTimes.put(damager.getUniqueId(), System.currentTimeMillis());
        }
        if (damageAmount < 1) {
            damageAmount = 1;
        }
        health.set(entity,
            health.get(entity) - damageAmount);
        if (health.get(entity) <= 0) {
            health.set(entity, 0);
        }
        if (damager != null && damager instanceof Player player) {
            RaxenPlayer rx = main.getManager().getPlayerManager().getRaxenPlayer(player);
            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_HURT, 0.65f, 0.75f);
        }
        if (health.get(entity) < 1) {
            die();
        } else {
            entity.playEffect(EntityEffect.HURT);
            if (mount != null) {
                mount.playEffect(EntityEffect.HURT);
            }
            displayCurrentHP();
        }
        lastDamaged = System.currentTimeMillis();
        return true;
    }

    public void displayCurrentHP() {
        entity.customName(getHPDisplay());
        Bukkit.getScheduler().scheduleSyncDelayedTask(main, lastScheduledRevertHP = new Runnable() {
            public void run() {
                if (!dead && entity != null && entity.isValid() && !entity.isDead()) {
                    if (lastScheduledRevertHP == this) {
                        entity.customName(Utils.colorComp(fullName));
                    }
                }
            }
        }, 60L);
    }

    public Component getHPDisplay() {
        EntityManager manager = main.getManager().getEntityManager();
        int health = manager.getHealth().get(entity);
        int maxHealth = manager.getMaxHealth().get(entity);
        double percent = ((double) health) / ((double) maxHealth);
        int green = (int) Math.floor(percent / 0.05);
        if (green > 20) {
            green = 20;
        } else if (green < 1) {
            green = 1;
        }
        StringBuilder sb = new StringBuilder();
        sb.append("<gray>");
        sb.append('[');
        int firstHalf = green - 10;
        if (firstHalf < 0) {
            firstHalf = 0;
        }

        sb.append("<dark_red>");
        for (int i = 0; i < 10 - firstHalf; i++) {
            sb.append('|');
        }
        sb.append("<green>");
        for (int i = 0; i < firstHalf; i++) {
            sb.append('|');
        }
        sb.append("<gold>"); // TODO: tier color
        sb.append(' ');
        sb.append(health);
        sb.append(' ');
        int secondHalf = green - firstHalf;

        sb.append("<dark_red>");
        for (int i = 0; i < 10 - secondHalf; i++) {
            sb.append('|');
        }
        sb.append("<green>");
        for (int i = 0; i < secondHalf; i++) {
            sb.append('|');
        }

        sb.append("<gray>");
        sb.append(']');
        return Utils.colorComp(sb.toString());
    }
}
