package me.unurled.raxen.components.entity.other;

import static me.unurled.raxen.utils.Utils.colorString;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.UUID;
import lombok.Getter;
import lombok.Setter;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.entity.DamageType;
import me.unurled.raxen.components.entity.player.RaxenPlayer;
import me.unurled.raxen.utils.Tags;
import net.kyori.adventure.text.Component;
import org.bukkit.Location;
import org.bukkit.NamespacedKey;
import org.bukkit.Sound;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

public abstract class RaxenEntity {

    private final Raxen main;

    @Getter
    private final String name;

    @Getter
    private final String ID;

    public Component fullName;

    @Getter
    private Integer level;

    @Getter
    @Setter
    private Double health;

    @Getter
    @Setter
    private Double strength;

    @Getter
    @Setter
    Boolean isAlive = false;

    boolean despawned = false;

    me.unurled.raxen.components.entity.other.custom.Entity entity;
    public HashMap<UUID, Long> lastAttackedByTimes;
    public java.util.HashMap<UUID, Long> damageDealt;

    @Getter
    @Setter
    Boolean isHostile;

    public RaxenEntity(
        Raxen main,
        String name,
        String id,
        Integer level,
        Double health,
        Double strength
    ) {
        this.main = main;
        this.name = name;
        ID = id;
        this.level = level;
        this.health = health;
        this.strength = strength;
        isHostile = true;
    }

    /*public Entity createEntity(
        me.unurled.raxen.components.entity.other.custom.Entity ent,
        String name,
        Location location
    ) {
        me.unurled.raxen.components.entity.other.custom.Entity e = (me.unurled.raxen.components.entity.other.custom.Entity) location.getWorld()
//            .spawnEntity(location, ent);
        e.setCustomNameVisible(true);
        e.customName(Component.text(name));
        entity = e;
        return e;
    }*/

    public void register(Entity e) {
        PersistentDataContainer data = e.getPersistentDataContainer();
        data.set(
            new NamespacedKey(main, "NAME"),
            PersistentDataType.STRING,
            name
        );
    }

    public Double getMaxHealth(Entity e) {
        PersistentDataContainer data = e.getPersistentDataContainer();
        EntityNamespacedKey key = new EntityNamespacedKey(main);
        return data.get(key.maxHealth, PersistentDataType.DOUBLE);
    }

    public abstract Entity createEntity(Location location);

    public void die() {
        die(true);
    }

    public void die(boolean drop) {
        if (!isAlive) {
            return;
        }
        isAlive = false;
        // set mob ai false
        Location deathLoc = entity.getLocation();
        entity.customName(fullName);
        Tags.makeFloatingText("<gold>+ EXP", deathLoc, 1.3, 1.0, 1.5, 1);
        // TODO: damge entity method damage
        // TODO: desawn
        // TODO: xp

        // xp
        int totalLevel = 0;
        int pdCount = 0;
        ArrayList<RaxenPlayer> players = new ArrayList<>();
        double damageSum = 0;
        RaxenPlayer player = null;
        for (Entry<UUID, Long> entry : damageDealt.entrySet()) {
            if ((player = main.getManager().getPlayerManager().getRaxenPlayer(entry.getKey()))
                != null) {
                damageSum += entry.getValue();
                totalLevel += player.getLevel();
                pdCount++;
                players.add(main.getManager().getPlayerManager().getRaxenPlayer(entry.getKey()));
            }
        }
        if (pdCount < 1) {
            totalLevel = 1;
            pdCount = 1;
        }
        double avgLv = totalLevel / (double) pdCount;
        for (RaxenPlayer pd : players) {
            if (damageDealt.containsKey(pd.getPlayer().getUniqueId())) {
                pd.incrementMobCounter(ID);
                double percentage =
                    damageDealt.get(pd.getPlayer().getUniqueId()) / (double) damageSum;
                if (percentage >= 0.10) {
                    pd.addMobKills();
                    // IDEA: is boss add to counter
                }
                long amt = (long) Math.ceil(entity.getXp() * percentage);
                if (!(level >= 80 && pd.getLevel() >= 80)
                    && Math.abs(pd.getLevel() - level) >= 10) {
                    double penalty = 0.70;
                    penalty -= (Math.abs(pd.getLevel() - level) - 10) * 0.06;
                    if (penalty < 0.10) {
                        penalty = 0.10;
                    }
                    pd.gainXp((long) Math.ceil(amt * penalty), true);
                } else {
                    pd.gainXp(amt);
                }
            }
        }
    }

    public void despawn() {
        if (despawned) {
            return;
        }
        despawned = true;
        if (entity != null) {
            entity.remove();
        }
        entity = null;
        // TODO: bosses, respawn etc
    }


    public boolean attack(RaxenPlayer pd) {
        return attack(pd, 0.5, -1, false);
    }

    public boolean attack(RaxenPlayer pd, int rpgDamage) {
        return attack(pd, 0.5, rpgDamage, false);
    }

    public boolean attack(RaxenPlayer pd, int rpgDamage, boolean projectile) {
        return attack(pd, 0.5, rpgDamage, projectile);
    }

    public boolean attack(RaxenPlayer pd, double knockback, int rDamage, boolean projectile) {
        boolean sucess = false;
        sucess = pd.damage(rDamage > -1 ? rDamage : (int) entity.getDamage(), entity,
            DamageType.NORMAL);
        if (sucess && !projectile) {
//            pd.knockback(entity, knockback);
            // TODO: knockback
        }
        return sucess;
    }

    public boolean damage(int damageAmount, Entity damager, DamageType damageType) {
        return damage(damageAmount, damager, damageType, false);
    }

    public boolean damage(int damage, Entity damager, DamageType damageType, boolean crit) {
        if (!isAlive) {
            return false;
        }
        if (damager != null && damager instanceof Player player) {
            if (!player.isOnline()
                || main.getManager().getPlayerManager().getRaxenPlayer(player) == null) {
                return false;
            }
        }
        if (damage > entity.getHealth()) {
            damage = entity.getHealth().intValue();
        }
        if (damager != null && damager instanceof Player player) {
            RaxenPlayer pd = main.getManager().getPlayerManager().getRaxenPlayer(player);
            if (damageDealt.containsKey(damager.getUniqueId())) {
                damageDealt.put(damager.getUniqueId(),
                    damageDealt.get(damager.getUniqueId()) + damage);
            } else {
                damageDealt.put(damager.getUniqueId(), (long) damage);
            }
        }
        if (damager != null) {
            lastAttackedByTimes.put(damager.getUniqueId(), System.currentTimeMillis());
        }
        if (damage < 1) {
            damage = 1;
        }
        health -= damage;
        if (health < 0) {
            health = 0.0;
        }
        if (damager != null && damager instanceof Player player) {
            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_HURT, 0.65f, 0.75f);
        }
        if (health <= 0) {
            die();
        } else {

        }
        return true;
    }

    public String getFullName() {
        if (isHostile) {
            return colorString(
                getLevel() +
                    "<white>⚔️" +
                    getName() +
                    "<red>" +
                    health +
                    "<white>/<white/>" +
                    getMaxHealth(entity)
            );
        } else {
            return colorString(
                getLevel() +
                    "<white>❤️" +
                    getName() +
                    "<red>" +
                    health +
                    "<white>/<white/>" +
                    getMaxHealth(entity)
            );
        }
    }
}
