package me.unurled.raxen.components.entity.player;

import static me.unurled.raxen.utils.Items.setItemsToInventory;
import static me.unurled.raxen.utils.Utils.debug;
import static me.unurled.raxen.utils.Utils.log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import lombok.Getter;
import lombok.Setter;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.entity.DamageType;
import me.unurled.raxen.components.entity.other.EntityNamespacedKey;
import me.unurled.raxen.components.entity.other.MobData;
import me.unurled.raxen.components.entity.player.attributes.Attribute;
import me.unurled.raxen.components.entity.player.storages.EnderChest;
import me.unurled.raxen.components.entity.player.storages.Inventory;
import me.unurled.raxen.components.party.Party;
import me.unurled.raxen.manager.entity.PlayerManager;
import me.unurled.raxen.manager.entity.SpellManager.CastState;
import me.unurled.raxen.utils.EntityAttributes;
import me.unurled.raxen.utils.Items;
import me.unurled.raxen.utils.Math;
import me.unurled.raxen.utils.Utils;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.EntityEffect;
import org.bukkit.Material;
import org.bukkit.Sound;
import org.bukkit.boss.BarColor;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BossBar;
import org.bukkit.entity.Entity;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

public class RaxenPlayer {

    private final Raxen main;
    @Getter
    private final Player player;
    private final EntityNamespacedKey namespacedKey;

    double crit_chance = 0.05;
    public double critDamage = 1.50;
    public boolean dead = false;

    private DamageType lastDamageCause = DamageType.NORMAL;

    private long lastEnvironmentalDamaged = 0;
    private long lastDamagedNonEnvironmental = 0;
    private long lastDamaged = 0;

    private Boolean isRaxenPlayer;

    private Attribute attribute;

    private Storage storages;
    private Inventories inventory;

    private boolean inTutorial;

    private HashMap<String, HashMap<String, Integer>> mobCounter = new HashMap<String, HashMap<String, Integer>>();

    private int mobKills, playerKills, deaths, bossKills;

    public long xp;
    @Getter
    private int level;

    @Getter
    @Setter
    private Party party = null;
    @Getter
    @Setter
    private Party invitedParty = null;

    BossBar bossBar;

    public RaxenPlayer(Raxen main, Player player) {
        this.main = main;
        this.player = player;
        this.namespacedKey = new EntityNamespacedKey(main);
        this.isRaxenPlayer = true;
    }

    public Attribute createNewAttribute() {
        debug(main, "Created new Attribute");
        EntityAttributes.setAttributes(
            player,
            100,
            100,
            0,
            50,
            0,
            100,
            0,
            100,
            0,
            100,
            100,
            0,
            10,
            0
        );
        return new Attribute(
            100,
            100,
            0,
            50,
            0,
            100,
            0,
            100,
            0,
            100,
            100,
            0,
            10,
            0
        );
    }

    public void setStorages(
        org.bukkit.inventory.Inventory inv,
        org.bukkit.inventory.Inventory ec
    ) {
        setStorage(ec);
        setInventory(inv);
    }

    public Storage getStorage() {
        PersistentDataContainer data = player.getPersistentDataContainer();
        String store = data.get(
            namespacedKey.storage,
            PersistentDataType.STRING
        );
        org.bukkit.inventory.Inventory ec = Bukkit.createInventory(
            null,
            54,
            Component.text("Ender Chest")
        );
        ec = setItemsToInventory(Items.mapItemStackDeserialize(store), ec);
        EnderChest ecc = new EnderChest(ec);
        Storage storage = new Storage(ecc);
        return storage;
    }

    public Inventories getInventory() {
        PersistentDataContainer data = player.getPersistentDataContainer();
        String inv = data.get(
            namespacedKey.inventory,
            PersistentDataType.STRING
        );
        org.bukkit.inventory.Inventory invv = Bukkit.createInventory(
            player,
            InventoryType.PLAYER
        );
        invv =
            setItemsToInventory(
                Items.listItemStackDeserialize(inv),
                (PlayerInventory) invv
            );
        Inventory invvv = new Inventory(invv);
        Inventories invvvv = new Inventories(invvv);
        return invvvv;
    }

    public void setInventory(org.bukkit.inventory.Inventory inv) {
        PersistentDataContainer data = player.getPersistentDataContainer();
        List<String> listInv = new ArrayList<String>();
        int reverse = 0;
        for (ItemStack it : inv) {
            reverse = reverse + 1;
            if (it != null && it.getType() != Material.AIR) {
                String s = Items.itemStackSerialize(it, reverse);
                listInv.add(s);
            }
        }
        String invv = Items.listItemStackSerialize(listInv);
        this.inventory = new Inventories(new Inventory(inv));
        data.set(namespacedKey.inventory, PersistentDataType.STRING, invv);
    }

    public void setStorage(org.bukkit.inventory.Inventory ec) {
        PersistentDataContainer data = player.getPersistentDataContainer();
        List<String> list = new ArrayList<String>();
        int reverse = 0;
        for (ItemStack it : ec) {
            if (it != null) {
                String s = Items.itemStackSerialize(it, reverse);
                list.add(s);
            }
            reverse += 1;
        }
        String store = Items.listItemStackSerialize(list);
        this.storages = new Storage(new EnderChest(ec));
        data.set(namespacedKey.storage, PersistentDataType.STRING, store);
    }

    public Storage createStorage() {
        setStorage(
            Bukkit.createInventory(null, 54, Component.text("Ender Chest"))
        );
        return getStorage();
    }

    public Inventories createInventory() {
        org.bukkit.inventory.Inventory inv = Bukkit.createInventory(null, InventoryType.PLAYER);
        setInventory(inv);
        return new Inventories(new Inventory(inv));
    }

    public Boolean isRaxenPlayer() {
        return isRaxenPlayer;
    }

    public void isRaxenPlayer(boolean player) {
        isRaxenPlayer = player;
    }

    public Attribute getAttribute() {
        return attribute;
    }

    public boolean isInTutorial() {
        return inTutorial;
    }

    public void setInTutorial(boolean inTutorial) {
        this.inTutorial = inTutorial;
    }

    public int getDamage() {
        return 0;
    }

    public boolean attackPlayer(RaxenPlayer player) {
        return attackPlayer(player, -1, false, 0.5);
    }

    public boolean attackPlayer(RaxenPlayer player, int damage) {
        return attackPlayer(player, damage, false, 0.5);
    }

    public boolean attackPlayer(RaxenPlayer player, int damage, boolean projectile) {
        return attackPlayer(player, damage, projectile, 0.5);
    }

    public boolean attackPlayer(RaxenPlayer player, int damage, boolean projectile,
        double knockback) {
        if (player == null) {
            return false;
        }
        if (player == null) {
            return false;
        }
        int eDamage;
        if (damage > 0) {
            eDamage = damage;
        } else {
            eDamage = getDamage();
        }
        boolean crit = false;
        if (java.lang.Math.random() < crit_chance) {
            crit = true;
            eDamage = (int) (eDamage * 1.5);
        }
        boolean sucess = false;
        // TODO: calculate with classes... later
        sucess = player.damage(eDamage, this.player, DamageType.NORMAL, crit);
        return sucess;
    }

    public boolean attackMob(MobData other) {
        return attackMob(other, 0.5, -1, false);
    }

    public boolean attackMob(MobData other, int rpgDamage) {
        return attackMob(other, 0.5, rpgDamage, false);
    }

    public boolean attackMob(MobData other, int rpgDamage, boolean projectile) {
        return attackMob(other, 0.5, rpgDamage, projectile);
    }

    public boolean attackMob(MobData other, double knockback, int rpgDamage,
        boolean projectil) {
        Player p = getPlayer();
        if (p == null || other == null) {
            return false;
        }
        int damage;
        if (rpgDamage > 0) {
            damage = rpgDamage;
        } else {
            damage = getDamage();
        }
        boolean crit = false;
        double critChanceTemp = crit_chance;
        if (java.lang.Math.random() < critChanceTemp) {
            crit = true;
            damage *= critDamage;
        }
        boolean sucess = false;
        sucess = other.damage(damage, p, DamageType.NORMAL, crit);
        if (sucess && !projectil) {
            // TODO: other.knockback(p, knockback);
        }
        return sucess;
    }

    public boolean damage(int damage, Entity damager, DamageType damageType) {
        return damage(damage, damager, damageType, false);
    }

    public boolean damage(int damage, Entity damager, DamageType damageType, boolean crit) {
        if (dead) {
            return false;
        }
        if (System.currentTimeMillis() - lastEnvironmentalDamaged < 600 && (
            damageType == DamageType.ENVIRONMENTAL_LAVA
                || damageType == DamageType.ENVIRONMENTAL_DROWNING
                || damageType == DamageType.ENVIRONMENTAL_FALL)) {
            return false;
        }
        if (player == null) {
            return false;
        }
        if (damager != null) {
            if (damager instanceof Player) {
                RaxenPlayer damagerRX = main.getManager().getPlayerManager()
                    .getRaxenPlayer((Player) damager);
                if (!((Player) damager).isOnline() || damagerRX == null) {
                    return false;
                }
                // TODO: check if same party later not now

            }
        }
        switch (lastDamageCause = damageType) {
            case NORMAL:
            case NORMAL_SPELL:
                damage -= Math.randInt(0, getAttribute().getDefense());
                lastDamagedNonEnvironmental = System.currentTimeMillis();
                break;
            case ENVIRONMENTAL_FALL:
                if (damage >= getAttribute().getHealth()) {
                    damage = getAttribute().getHealth() - 1;
                }
                break;
        }
        if (damageType == DamageType.ENVIRONMENTAL_LAVA
            || damageType == DamageType.ENVIRONMENTAL_DROWNING
            || damageType == DamageType.ENVIRONMENTAL_FALL) {
            lastEnvironmentalDamaged = System.currentTimeMillis();
        } else if (damageType == DamageType.NORMAL) {
            lastDamaged = System.currentTimeMillis();
        }
        if (damager != null && damager != player && damager instanceof Player) {
            damage *= 0.8; // pvp nerf
        }
        if (damage < 1) {
            damage = 1;
        }
        if (damageType == DamageType.NORMAL_SPELL || damageType == DamageType.NORMAL) {
            if (damage > getAttribute().getHealth() * 0.5) {
                player.leaveVehicle();
            }
        }
        getAttribute().setHealth(getAttribute().getHealth() - damage);
        if (getAttribute().getHealth() < 0) {
            getAttribute().setHealth(0);
        }
        if (damager != null && damager != player) {
            if (damager instanceof Player) {
                RaxenPlayer damagerRX = main.getManager().getPlayerManager()
                    .getRaxenPlayer((Player) damager);
                if (damagerRX != null) {
                    Player p2 = damagerRX.getPlayer();
                    if (p2 != null && p2.isValid()) {
                        p2.playSound(p2.getLocation(), Sound.ENTITY_PLAYER_HURT, 0.65f, 0.75f);
                    }
                }
            }
        }
        player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_HURT, 0.85f, 0.85f);
        if (System.currentTimeMillis() - lastDamaged < 1000) {
            player.playEffect(EntityEffect.HURT);
        }
        if (getAttribute().getHealth() <= 0) {
            die();
        } else {
            // TODO: update health display on action bar
        }
        return true;
    }

    public void die() {
        log("player " + player.displayName() + " died");
    }

    public void incrementMobCounter(String mob) {
        if (mobCounter.containsKey(mob)) {
            mobCounter.get(mob).replaceAll((k, v) -> v + 1);
            // TODO: mob quest tracker ?
        }
    }

    public int getMobKills() {
        return mobKills;
    }

    public int getPlayerKills() {
        return playerKills;
    }

    public int getDeaths() {
        return deaths;
    }

    public int getBossKills() {
        return bossKills;
    }

    public void setMobKills(int mobKills) {
        this.mobKills = mobKills;
    }

    public void setPlayerKills(int playerKills) {
        this.playerKills = playerKills;
    }

    public void setDeaths(int deaths) {
        this.deaths = deaths;
    }

    public void setBossKills(int bossKills) {
        this.bossKills = bossKills;
    }

    public void addMobKills(int mobKills) {
        this.mobKills += mobKills;
    }

    public void addMobKills() {
        this.mobKills += 1;
    }

    public void addPlayerKills(int playerKills) {
        this.playerKills += playerKills;
    }

    public void addPlayerKills() {
        this.playerKills += 1;
    }

    public void addDeaths(int deaths) {
        this.deaths += deaths;
    }

    public void addDeaths() {
        this.deaths += 1;
    }

    public void addBossKills(int bossKills) {
        this.bossKills += bossKills;
    }

    public void addBossKills() {
        this.bossKills += 1;
    }

    public void gainXp(long amt) {
        gainXp(amt, false);
    }

    public void gainXp(long amt, boolean penalty) {
        xp += amt;
        Player p = getPlayer();
        if (p == null || !p.isValid()) {
            return;
        }
        int bonus = 0;
        // IDEA: per class xp bonus
        // IDEA: party xp multiplier
        long tXp = amt;
        if (bonus > 0) {
            tXp += bonus;
            xp += bonus;
        }
        if (xp >= getXpForNextLevel()) {
            long extra = xp - getXpForNextLevel();
            xp = 0;
            level++;
            xp = extra < getXpForNextLevel() ? extra : getXpForNextLevel() - 1;
            // TODO: take care of stats level up
            // TODO: update health and mana display;
        }
    }

    private int lastLevel = -1;
    private long cachedXp = -1;

    public long getXpForNextLevel() {
        long ret = 1;
        if (lastLevel == level && cachedXp > 0) {
            return cachedXp;
        }
        ret = PlayerStatistics.getEXPForNextLevel(level);
        if (ret < 0) {
            ret = Long.MAX_VALUE;
        }
        lastLevel = level;
        cachedXp = ret;
        return ret;
    }

    public void knockback(LivingEntity attacker, double knockBackMultiplier) {
        return;
    }

    public void updateHealthManaDisplay() {
        Player p = getPlayer();
        PlayerManager manager = main.getManager().getPlayerManager();
        if (p != null) {
            int health = manager.getHealth().get(p);
            int maxHealth = manager.getMaxHealth().get(p);
            int mana = manager.getMana().get(p);
            int maxMana = manager.getMaxMana().get(p);
            if (health > maxHealth) {
                manager.getHealth().set(p, maxHealth);
                health = manager.getHealth().get(p);
            }
            if (health < 0) {
                manager.getHealth().set(p, 0);
                health = manager.getHealth().get(p);
            }
            if (mana > maxMana) {
                manager.getMana().set(p, maxMana);
                mana = manager.getMana().get(p);
            }
            if (dead) {
                return;
            }
            double percent = ((double) health) / ((double) maxHealth);
            StringBuilder sb = new StringBuilder();
            BarColor color = BarColor.GREEN;
            if (percent > 0.5) {
                sb.append("<green>");
                sb.append(health);
                sb.append(" <aqua><bold>HP");
            } else if (percent > 0.20) {
                sb.append("<yellow>");
                sb.append(health);
                sb.append(" <aqua><bold>HP");
                color = BarColor.YELLOW;
            } else {
                sb.append("<dark_red><bold>DANGER - <reset><red>");
                sb.append(health);
                sb.append(" <dark_red><bold> - DANGER");
                color = BarColor.RED;
            }
            sb.append("<reset>");
            // TODO: status poison, burn, bleed
            if (percent > 1) {
                percent = 1;
            } else if (percent < 0.01) {
                percent = 0.01;
            }
            if (bossBar == null) {
                bossBar = Bukkit.createBossBar(Utils.coloredString(sb.toString()), BarColor.GREEN,
                    BarStyle.SOLID);
                bossBar.addPlayer(p);
            }
            bossBar.setTitle(Utils.coloredString(sb.toString().trim()));
            bossBar.setProgress(percent);
            bossBar.setColor(color);

            if (!p.isGliding() && main.getManager().getSpellManager().casters.containsKey(
                p.getUniqueId())) {
                CastState cs = main.getManager().getSpellManager().casters.get(
                    p.getUniqueId()).state;
                if (cs != CastState.NONE) {
                    StringBuilder sb2 = new StringBuilder();
                    sb2.append("<gold><bold> ");
                    sb2.append(cs.toString());
                    p.sendActionBar(Utils.colorComp(sb2.toString()));
                }
            }

//            p.setFoodLevel(mana * 2);
            p.setLevel(level);
            p.setExp((float) (((double) xp) / ((double) getXpForNextLevel())));
        }
    }
}
