package me.unurled.raxen.components.items;

import com.google.common.collect.Multimap;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import me.unurled.raxen.utils.Utils;
import net.kyori.adventure.text.Component;
import org.bukkit.Color;
import org.bukkit.Material;
import org.bukkit.attribute.Attribute;
import org.bukkit.attribute.AttributeModifier;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.FireworkEffectMeta;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.LeatherArmorMeta;
import org.bukkit.inventory.meta.MapMeta;
import org.bukkit.inventory.meta.PotionMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.potion.PotionData;
import org.bukkit.potion.PotionEffect;
import org.jetbrains.annotations.NotNull;

public class ItemBuilder {

    private final PersistentDataContainer persistentDataContainer;
    private ItemStack it;
    private Material type;
    private Component displayName;
    private int amount;
    private int customModelData;
    private String customModelDataPath;
    private boolean hasCustomModelData;
    private Color color;
    private @NotNull PotionData potionData;

    private List<PotionEffect> potionEffects;
    private Set<ItemFlag> itemFlags;
    private boolean hasAttributeModifiers;
    private Multimap<Attribute, AttributeModifier> attributeModifiers;
    private List<String> lore;
    private final Map<Enchantment, Integer> enchantments;

    public ItemBuilder(ItemStack it) {
        this.it = it;
        type = it.getType();
        amount = it.getAmount();

        final ItemMeta itm = it.getItemMeta();

        if (itm instanceof LeatherArmorMeta leatherArmorMeta) {
            color = leatherArmorMeta.getColor();
        }

        if (itm instanceof PotionMeta potionMeta) {
            color = potionMeta.getColor();
            potionData = potionMeta.getBasePotionData();
            potionEffects = new ArrayList<>(potionMeta.getCustomEffects());
        }

        if (itm instanceof MapMeta mapMeta) {
            color = mapMeta.getColor();
        }

        if (itm instanceof FireworkEffectMeta effectMeta) {
            color =
                effectMeta.hasEffect() ? effectMeta.getEffect().getColors().get(0) : Color.WHITE;
        }

        if (itm.hasDisplayName()) {
            displayName = itm.displayName();
        }

        if (!itm.getItemFlags().isEmpty()) {
            itemFlags = itm.getItemFlags();
        }

        hasAttributeModifiers = itm.hasAttributeModifiers();
        if (hasAttributeModifiers) {
            attributeModifiers = itm.getAttributeModifiers();
        }

        hasCustomModelData = itm.hasCustomModelData();
        if (itm.hasCustomModelData()) {
            customModelData = itm.getCustomModelData();
        }

        if (itm.hasLore()) {
            lore = itm.getLore();
        }

        persistentDataContainer = itm.getPersistentDataContainer();

        enchantments = new HashMap<>();
    }

    public ItemBuilder(Material material) {
        this(new ItemStack(material));
    }

    public ItemBuilder displayName(String name) {
        this.displayName = Utils.colorComp(name);
        return this;
    }

    public void lore(List<String> lore) {
        this.lore = lore;
    }

    public void color(Color color) {
        this.color = color;
    }

    public void modelData(String path, int data) {
        this.customModelData = data;
        this.customModelDataPath = path;
    }
}
