package me.unurled.raxen.components.party;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.UUID;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.entity.player.RaxenPlayer;
import me.unurled.raxen.manager.entity.PartyManager;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.ClickEvent.Action;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.Bukkit;
import org.bukkit.Sound;
import org.bukkit.entity.Player;
import org.bukkit.scoreboard.Scoreboard;

public class Party {

    Raxen main;

    private static int ID = 1;
    public int id = ID++;

    private ArrayList<UUID> uuids = new ArrayList<UUID>();
    private UUID leaderUUID;
    private String leaderName;

    private ArrayList<Player> cachedPlayer = new ArrayList<Player>();

    private long lastPlayerUpdate = 0L;
    private Scoreboard board;

    private boolean lootShare = false;
    private HashMap<String, Long> lastInv = new HashMap<String, Long>();


    public Party(Raxen main, Player leader) {
        this.main = main;
        this.leaderUUID = leader.getUniqueId();
        this.leaderName = leader.getName();
        // IDEA: adds scoreboard
        addPlayer(leader);
    }

    public void sendInvite(Player p, RaxenPlayer rx, Player inviter) {
        if (rx.getParty() == this) {
            inviter.sendMessage(PartyManager.PREFIX.append(
                Component.text(p.getName() + " is already in the party!")));
            return;
        }
        if (lastInv.containsKey(p.getName())) {
            if (System.currentTimeMillis() - lastInv.get(p.getName()) < 15000) {
                inviter.sendMessage(PartyManager.PREFIX.append(Component.text(p.getName()
                        + " was invited to the party in the last 15 seconds. Please wait a bit before inviting them again.")
                    .color(
                        TextColor.color(255, 0, 0))));
                return;
            }
        }
        lastInv.put(p.getName(), System.currentTimeMillis());
        p.playSound(p.getLocation(), Sound.ENTITY_EXPERIENCE_ORB_PICKUP, 10.0F, 1.0F);
        p.sendMessage(PartyManager.PREFIX.append(Component.text(
            "You have been invited to join " + leaderName
                + "'s party. Type /party join to join the party.").color(
            TextColor.color(0, 255, 0))));
        p.sendMessage(
            Component.text("Or click Here").color(TextColor.color(255, 255, 0)).clickEvent(
                ClickEvent.clickEvent(Action.SUGGEST_COMMAND, "/party join")));
        p.sendMessage(PartyManager.PREFIX.append(
            Component.text("Type /party decline to decline the party invitation.").color(
                TextColor.color(0, 255, 0))));
        p.sendMessage(
            Component.text("Or click Here").color(TextColor.color(255, 255, 0)).clickEvent(
                ClickEvent.clickEvent(Action.SUGGEST_COMMAND, "/party decline")));
        sendMessage(Component.text(
            p.getName() + " has been invited to the party by " + inviter.getName() + "."));
    }

    public void addPlayer(Player p) {
        if (isFull()) {
            p.sendMessage(PartyManager.PREFIX.append(Component.text("This party is full!")));
            sendMessage(Component.text(p.getName() + " tried to join the party, but it was full."));
            return;
        }
        lastPlayerUpdate = 0;
        uuids.add(p.getUniqueId());
        RaxenPlayer rx = main.getManager().getPlayerManager()
            .getRaxenPlayer(p);
        if (rx != null) {
            rx.setParty(this);
        }
        p.sendMessage(Component.text("You have joined " + leaderName + "'s the party."));
        Component comp = Component.text("The party is composed of ");
        for (Player p2 : getPlayers()) {
            comp.append(Component.text(p2.getName() + ", "));
        }
        p.sendMessage(comp);
        sendMessage(Component.text(p.getName() + " has joined the party."));
    }

    public void leavePlayer(UUID uuid, String name) {
        if (uuid != null) {
            return;
        }
        Player player = Bukkit.getPlayer(uuid);
        if (leaderUUID != null && leaderUUID == uuid) {
            sendMessage(Component.text(name + " has left the party."));
            if (player != null && player.isOnline()) {
                main.getManager()
                    .getPlayerManager().getRaxenPlayer(player).setParty(null);
            }
            uuids.remove(uuid);
            lastPlayerUpdate = 0;
            if (uuids.size() == 0) {
                destroy();
            } else {
                for (int k = 0; k < uuids.size(); k++) {
                    UUID nu = uuids.get(k);
                    Player temp = Bukkit.getPlayer(nu);
                    if (temp != null && temp.isOnline()) {
                        sendMessage(Component.text(temp.getName() + " is now the party leader."));
                        leaderUUID = nu;
                        leaderName = temp.getName();
                        break;
                    }
                }
            }
        } else {
            if (player != null && player.isOnline()) {
                sendMessage(Component.text(name + " has left the party."));
                main.getManager()
                    .getPlayerManager().getRaxenPlayer(player).setParty(null);
            }
            uuids.remove(uuid);
            lastPlayerUpdate = 0;
            if (uuids.size() == 0) {
                destroy();
            }
        }
    }

    public boolean isFull() {
        if (this.uuids == null) {
            return true;
        }
        return uuids.size() >= PartyManager.MAX_SIZE;
    }

    public double getXpMultiplier() {
        int size = uuids.size();
        if (size < 0) {
            size = 0;
        }
        return switch (size) {
            case 0, 1 -> 0.0;
            case 2 -> 0.1;
            case 3 -> 0.15;
            case 4 -> 0.2;
            default -> 0.25;
        };
    }

    public void destroy() {
        for (UUID uuid : uuids) {
            if (main.getManager().getPlayerManager().getRaxenPlayer(uuid) != null) {
                RaxenPlayer rx = main.getManager().getPlayerManager().getRaxenPlayer(uuid);
                if (rx.getParty() != null && rx.getParty().equals(this)) {
                    rx.setParty(null);
                }
            }
            cachedPlayer.clear();
            cachedPlayer = null;
            uuids.clear();
            uuids = null;
            leaderUUID = null;
            leaderName = null;
            board = null;
            main.getManager().getPartyManager().getParties().remove(this);
        }
    }

    public ArrayList<Player> getPlayers() {
        if (System.currentTimeMillis() - lastPlayerUpdate > 5000) {
            lastPlayerUpdate = System.currentTimeMillis();
            cachedPlayer.clear();
            ArrayList<UUID> toRemove = new ArrayList<UUID>();
            for (UUID uuid : uuids) {
                Player p = Bukkit.getPlayer(uuid);
                if (p != null && p.isOnline()) {
                    cachedPlayer.add(p);
                } else {
                    toRemove.add(uuid);
                }
            }
            uuids.removeAll(toRemove);
            return cachedPlayer;
        } else {
            for (int k = 0; k < cachedPlayer.size(); k++) {
                Player p = cachedPlayer.get(k);
                if (p == null || !p.isOnline()) {
                    cachedPlayer.remove(k);
                    k--;
                    if (k < 0) {
                        k = 0;
                    }
                }
            }
            return cachedPlayer;
        }
    }

    public void sendMessage(Player sender, Component comp) {
        Component msg = PartyManager.PREFIX.append(
            Component.text(sender.getName() + "<reset><white>: ")).append(comp);
        // TODO: Filter words
        for (Player p : getPlayers()) {
            p.sendMessage(msg);
        }
    }

    public void sendMessage(Component comp) {
        Component msg = PartyManager.PREFIX.append(comp);
        for (Player p : getPlayers()) {
            p.sendMessage(msg);
            p.playSound(p.getLocation(), Sound.ENTITY_EXPERIENCE_ORB_PICKUP, 1, 1);
        }
    }

    public boolean isLeader(Player p) {
        return p.getName().equals(leaderName) && p.getUniqueId().equals(leaderUUID);
    }
}
