package me.unurled.raxen.components.resourcepack.hosting;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import com.google.gson.JsonSyntaxException;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.UUID;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;

public class PackHoster implements Hosting {

    private final String serverAddress;
    private String packURL;
    private String minecraftPackURL;
    private String sha1;
    private UUID uuid;

    public PackHoster(String serverAddress) {
        this.serverAddress = "https://" + serverAddress + "/";
    }

    @Override
    public boolean upload(File resoucepack) {
        try {
            HttpClient httpClient = HttpClientBuilder.create().build();

            HttpPost httpPost = new HttpPost(serverAddress + "upload/" + uuid.toString());

            HttpEntity entity = MultipartEntityBuilder.create()
                .addBinaryBody("file", resoucepack, ContentType.DEFAULT_BINARY,
                    resoucepack.getName())
                .build();
            httpPost.setEntity(entity);

            // Execute the request
            HttpResponse response = null;
            response = httpClient.execute(httpPost);
            String responseString = EntityUtils.toString(response.getEntity());
            JsonObject json;
            try {
                json = JsonParser.parseString(responseString).getAsJsonObject();
            } catch (JsonSyntaxException e) {
                return false;
            }

            if (json.has("url") && json.has("sha1")) {
                packURL = json.get("url").getAsString();
                minecraftPackURL = packURL.replace("https://", "http://");
                sha1 = json.get("sha1").getAsString();
                return true;
            }

            // Get the response code
            int responseCode = response.getStatusLine().getStatusCode();
            System.out.println("Response Code: " + responseCode);

            // Read the response body
            String responseBody = null;

            responseBody = EntityUtils.toString(response.getEntity(), StandardCharsets.UTF_8);
            System.out.println("Response Body: " + responseBody);
        } catch (IOException e) {
            return false;
        }

        return true;
    }

    @Override
    public UUID getUUID() {
        return uuid;
    }

    public void setUUID(UUID uuid) {
        this.uuid = uuid;
    }

    @Override
    public String getPackURL() {
        return packURL;
    }

    @Override
    public String getMinecraftPackURL() {
        return minecraftPackURL;
    }

    @Override
    public byte[] getSHA1() {
        int len = sha1.length();
        byte[] data = new byte[len / 2];
        for (int i = 0; i < len; i += 2) {
            data[i / 2] = (byte) ((Character.digit(sha1.charAt(i), 16) << 4)
                + Character.digit(sha1.charAt(i + 1), 16));
        }
        return data;
    }

    @Override
    public String getOriginalSHA1() {
        return sha1;
    }
}
