package me.unurled.raxen.config;

import java.io.File;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.items.ItemBuilder;
import me.unurled.raxen.components.items.ItemParser;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.YamlConfiguration;

public class ItemConfig {

    private File itemsFolder;
    private Raxen main;

    public ItemConfig(Raxen main) {
        this.main = main;
        validatesConfig();
    }

    public void validatesConfig() {
        itemsFolder = new File(main.getDataFolder(), "items");
        if (!itemsFolder.exists()) {
            itemsFolder.mkdirs();
        }
    }

    public Map<File, Map<String, ItemBuilder>> parseItemConfigs() {
        Map<File, Map<String, ItemBuilder>> parseMap = new LinkedHashMap<>();
        List<File> config = Arrays.stream(getItemsFiles())
            .filter(file -> file.getName().endsWith(".yml")).toList();
        for (File file : config) {
            parseMap.put(file, parseItemConfig(YamlConfiguration.loadConfiguration(file), file));
        }
        return parseMap;
    }

    public Map<String, ItemBuilder> parseItemConfig(YamlConfiguration config, File file) {
        Map<String, ItemParser> parseMap = new LinkedHashMap<>();
        for (String itemSelection : config.getKeys(false)) {
            if (!config.isConfigurationSection(itemSelection)) {
                continue;
            }
            ConfigurationSection itemSection = config.getConfigurationSection(itemSelection);
            parseMap.put(itemSelection, new ItemParser(itemSection));
        }
        boolean configUpdated = false;
        Map<String, ItemBuilder> map = new LinkedHashMap<>();
        for (Map.Entry<String, ItemParser> entry : parseMap.entrySet()) {
            ItemParser parser = entry.getValue();
            try {
                map.put(entry.getKey(), parser.buildItem());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return map;
    }

    /*public Map<File, Map<String, EffectBuilder>> parseEffectConfig() {
        // TODO: efffects
    }*/

    private File[] getItemsFiles() {
        if (!itemsFolder.exists()) {
            itemsFolder.mkdirs();
            return new File[0];
        }
        File[] itemConfigs = Arrays.stream(itemsFolder.listFiles()).filter(File::isFile)
            .toArray(File[]::new);
        Arrays.sort(itemConfigs);
        return itemConfigs;
    }
}
