package me.unurled.raxen.utils;

import eu.decentsoftware.holograms.api.DHAPI;
import eu.decentsoftware.holograms.api.holograms.Hologram;
import eu.decentsoftware.holograms.api.holograms.HologramLine;
import eu.decentsoftware.holograms.api.holograms.HologramPage;
import eu.decentsoftware.holograms.api.utils.Common;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import me.unurled.raxen.Raxen;
import me.unurled.raxen.components.entity.other.RaxenEntity;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.TextComponent;
import org.apache.commons.lang.Validate;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.util.Vector;

public class Tags {

    public static final double STEP = 0.5;

    public static void makeFloatingText(String name, Location loc, double xzOffset, double yMin,
        double yMax, double durationSec) {
        loc.add(-xzOffset / 2 + (java.lang.Math.random() * (xzOffset)),
            (java.lang.Math.random() * (yMax - yMin)) + yMin, -xzOffset / 2 + (
                java.lang.Math.random() * (xzOffset)));
        DHAPI.createHologram(name, loc, List.of(name));
        new BukkitRunnable() {
            @Override
            public void run() {
                DHAPI.removeHologram(name);
            }
        }.runTaskLater(((Raxen) Objects.requireNonNull(
            Bukkit.getPluginManager()
                .getPlugin("Raxen"))).getPlugin(), (long) (durationSec * 20));
    }

    public static void makeFloatingTextStylishWhenHit(Location damager, Location victim,
        String name, String... text) {
        ArrayList<Location> loc = getParabolicCurve(victim, damager);
        Hologram holo = DHAPI.createHologram(name, victim, Arrays.stream(text).toList());
        for (Location location : loc) {
            DHAPI.moveHologram(holo, location);
            if (loc.indexOf(location) == loc.size() - 1) {
                new BukkitRunnable() {
                    @Override
                    public void run() {
                        DHAPI.removeHologram(name);
                    }
                }.runTaskLater(((Raxen) Objects.requireNonNull(
                    Bukkit.getPluginManager()
                        .getPlugin("Raxen"))).getPlugin(), 20);
            }
        }
    }

    public static ArrayList<Location> getParabolicCurve(Location start, Location end) {
        ArrayList<Location> results = new ArrayList<>();

        Vector line = end.clone().subtract(start).toVector();
        line.setY(start.getY());
        double length = line.length();
        line.normalize();

        for (double x = 0; x < length; x += STEP) {
            double a = -1.0 / x;
            double y = a * java.lang.Math.pow(x, 2) + x;
            Location point = start.clone();
            point.add(line.multiply(x)).setY(point.getY() + y);
            results.add(point);
            line.multiply(1 / x);
        }

        return results;
    }

    public static Hologram createNameTag(Raxen main, Entity entity) {
        RaxenEntity e = main.getManager().getEntityManager().getEntity(entity);

        // TODO: implement this...

        if (e != null) {
            TextComponent txt = Utils.colorTextComp(e.getFullName());
            return createHologram(e.getName() + UUID.randomUUID(), entity.getLocation(),
                false, txt);
        } else {
            // register entity...
        }
        return null;
    }

    public static Hologram createHologram(String name, Location location, boolean saveToFile,
        TextComponent lines) throws IllegalArgumentException {
        Validate.notNull(name);
        Validate.notNull(location);

        if (!name.matches(Common.NAME_REGEX)) {
            throw new IllegalArgumentException(String.format(
                "Hologram name can only contain alphanumeric characters, underscores and dashes! (%s)",
                name));
        }

        if (Hologram.getCachedHologramNames().contains(name)) {
            throw new IllegalArgumentException(
                String.format("Hologram with that name already exists! (%s)", name));
        }

        Hologram hologram = new Hologram(name, location, saveToFile);
        HologramPage page = hologram.getPage(0);
        if (lines != null) {
            for (Component line : ComponentSplit.split(lines, Component.newline())) {
                HologramLine hologramLine = new HologramLine(page, page.getNextLineLocation(),
                    Utils.textCompToString(line));
                page.addLine(hologramLine);
            }
        }
        hologram.showAll();
        hologram.save();
        return hologram;
    }

}
